/*
 *   Copyright (C) 2002-2006
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include <fstream>
#include <string>
#include <stdarg.h>
#include "config.h"

using namespace std;

#include "Log.h"

#ifdef SYS_SOLARIS
#include <strings.h>
#endif

Log::Log() {
	debug = 0;
}

void Log::setlogfile( char *file ) {
	logfile = file;
}

void Log::init() {
	if (logfile.empty())
		logfile = DEFAULT_LOG_FILE;
	report("Logging started\n");
}

#ifdef __STDC__
int Log::report(char *fmt, ...)
#else
int Log::report(char *fmt, va_dcl)
#endif
{
	char *msg = new char[1024];
	va_list ap;
	fstream log;

#ifdef __STDC__
	va_start(ap, fmt);
#else
	va_start(ap);
#endif

	vsnprintf(msg, 1024, fmt, ap);
	va_end(ap);

	if (debug) {
		log.open(logfile.c_str(), fstream::out | fstream::app);
		if (log.is_open()) {
			time_t t = time(NULL);
			log << string(ctime(&t)) +msg;
			log.close();
		} else {
			cerr << "Cannot open logfile " << logfile << endl;
		}
	}

	delete msg;
}

int Log::report(string *msg)
{
	fstream log;

	log.open(logfile.c_str(), fstream::out | fstream::app);
	if (log.is_open()) {
		time_t t = time(NULL);
		log << string(ctime(&t)) + msg->c_str();
		log << "\n";
		log.close();
	} else {
		cerr << "Cannot open logfile " << logfile << endl;
	}
}
