/*
 *   Copyright (C) 2002-2006
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <netinet/in.h>
#include "tac_plus.h"
#include "tacacs.h"
#include "md5.h"

tauthen::tauthen(tacacs *session) {
	sock = session->sock;
	dbh = session->dbh;
	fileh = session->fileh;
	type = new struct authen_type;
	aborted = 0;
	type->authen_func = 0;
	state = STATE_AUTHEN_START;
}

tauthen::~tauthen() {
	if (client_data != NULL) delete[] client_data;
	delete type;
}

int tauthen::choose()
{
	int iterations = 0, status;
	string prompt, msg;
	u_char *reply, *p;
	struct authen_cont *cont;
	struct identity *indentp;

	if (taclog->debug & DEBUG_AUTHEN_FLAG)
		taclog->report("tauthen::choose - Choose authentication\n");

	while (++iterations <= TAC_PLUS_MAX_ITERATIONS) {
		switch (choose_authen()) {
		case CHOOSE_BADTYPE:  /* FIXME */
		default:
			sock->send_authen_error(string("tauthen::choose - unexpected failure return"));
			return (1);

		case CHOOSE_OK:
			return (0);

		case CHOOSE_FAILED:
			sock->send_authen_error(string("tauthen::choose - unacceptable authen method"));
			return (1);

		case CHOOSE_GETUSER:
			HDR *hdr1;

			status = TAC_PLUS_AUTHEN_STATUS_GETUSER;
			if (service == TAC_PLUS_AUTHEN_SVC_LOGIN) {
				dbh->get_host("prompt",id->NAS_addr, &prompt);
			} 
			prompt.append(PROMPT_INSERT);
			sock->send_authen_reply(TAC_PLUS_AUTHEN_STATUS_GETUSER,
						prompt.c_str(),
						server_data.c_str(),
						0 );

			if (taclog->debug & DEBUG_AUTHEN_FLAG)
				taclog->report("tauthen::choose - sent(%s)\n",prompt.c_str());

//			if ((reply=sock->get_authen_cont())==NULL) {
			if (sock->get_authen_cont(&reply)) {
				return(1);
			}
			hdr1 = (HDR*) reply;
			cont = (struct authen_cont *) (reply + TAC_PLUS_HDR_SIZE);
			if (cont->flags & TAC_PLUS_CONTINUE_FLAG_ABORT) {
				string pMsg;

				if (cont->user_data_len) {
					p = reply + TAC_PLUS_HDR_SIZE + TAC_AUTHEN_CONT_FIXED_FIELDS_SIZE + cont->user_msg_len;
					pMsg = string((char*)p).substr(0,cont->user_data_len);
				}

				if (taclog->debug & DEBUG_AUTHEN_FLAG)
					taclog->report("tauthen::choose - Login aborted by request -- msg: %s\n", pMsg.c_str());
				return (1);
			}

			p = reply + TAC_PLUS_HDR_SIZE + TAC_AUTHEN_CONT_FIXED_FIELDS_SIZE;
			if (!id->username->empty()) delete id->username;
			id->username = new string(string((char*)p).substr(0,cont->user_msg_len));
			if (taclog->debug & DEBUG_AUTHEN_FLAG)
				taclog->report("tauthen::choose - got username(%s)\n", id->username->c_str());
			delete reply;
		}
	}

	if (taclog->debug & DEBUG_AUTHEN_FLAG)
		taclog->report("tauthen::choose - Too many iterations\n");
	return (1);
}

int tauthen::choose_authen()
{
	switch (action) {
	case TAC_PLUS_AUTHEN_SENDPASS:
		switch (type->authen_type) {
		case TAC_PLUS_AUTHEN_TYPE_CHAP:
		case TAC_PLUS_AUTHEN_TYPE_MSCHAP:
		case TAC_PLUS_AUTHEN_TYPE_PAP:
		case TAC_PLUS_AUTHEN_TYPE_ARAP:
			if (taclog->debug & DEBUG_AUTHEN_FLAG)
				taclog->report("tauthen::choose_authen - authen_type %u\n", type->authen_type);
			if (sock->version != TAC_PLUS_VER_0)
				break;

			if (id->username->empty())
				return (CHOOSE_GETUSER);

			type->authen_func = SENDPASS_FN;
			return (CHOOSE_OK);

		default:
			break;
		}
		break;

	case TAC_PLUS_AUTHEN_SENDAUTH:
		switch (type->authen_type) {
		case TAC_PLUS_AUTHEN_TYPE_MSCHAP:
#ifndef MSCHAP_DES
			sock->version = TAC_PLUS_VER_0;
			if (taclog->debug & DEBUG_AUTHEN_FLAG)
				taclog->report("tauthen::choose_authen - DES is unvailable\n");
			return (CHOOSE_FAILED);
#endif
		case TAC_PLUS_AUTHEN_TYPE_CHAP:
		case TAC_PLUS_AUTHEN_TYPE_PAP:
			//must version 1
			if (sock->version != TAC_PLUS_VER_1)
				break;

			if (id->username->empty())
				return (CHOOSE_FAILED);

			type->authen_func = SENDAUTH_FN;
			return (CHOOSE_OK);

		default:
			break;
		}
		break;

	case TAC_PLUS_AUTHEN_LOGIN:
		//For enabling, enable_fn handles everything. Must be minor version zero
		if (service == TAC_PLUS_AUTHEN_SVC_ENABLE) {
			if (sock->version != TAC_PLUS_VER_0) {
				break;
			}

			if (id->username->empty()) {
				return (CHOOSE_GETUSER);
			}

			type->authen_func = ENABLE_FN;
			return (CHOOSE_OK);
		}

		switch (type->authen_type) {
		case TAC_PLUS_AUTHEN_TYPE_ASCII:
			if (sock->version != TAC_PLUS_VER_0) {
				break;
			}

			if (id->username->empty()) {
				return (CHOOSE_GETUSER);
			}
#ifdef SKEY
			if (skey = get_skey()) {
				type->authen_func = skey_fn;
				strcpy(type->authen_name, "skey_fn");
				return (CHOOSE_OK);
			}
#endif
			type->authen_func = DEFAULT_FN;
			return (CHOOSE_OK);

		case TAC_PLUS_AUTHEN_TYPE_ARAP:
		case TAC_PLUS_AUTHEN_TYPE_MSCHAP:
#if !defined(ARAP_DES) || !defined(MSCHAP_DES)
			sock->version = TAC_PLUS_VER_0;
			if (taclog->debug & DEBUG_AUTHEN_FLAG)
				taclog->report("tauthen::choose_authen - DES unvailable\n");
			return (CHOOSE_FAILED);
#endif
		case TAC_PLUS_AUTHEN_TYPE_PAP:
		case TAC_PLUS_AUTHEN_TYPE_CHAP:
			if (sock->version == TAC_PLUS_VER_0) {
				type->authen_func = DEFAULT_V0_FN;
				return (CHOOSE_OK);
			}

			if (id->username->empty()) {
				if (taclog->debug & DEBUG_AUTHEN_FLAG)
					taclog->report("tauthen::choose_authen - No user in START packet for PAP/CHAP/ARAP\n");
				return (CHOOSE_FAILED);
			}

			type->authen_func = DEFAULT_FN;
			return (CHOOSE_OK);

		default:
			break;
		}
		break;

	case TAC_PLUS_AUTHEN_CHPASS:
		if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::choose_authen - Change password is not supported in this version\n");
		/** not supported now */
		return (CHOOSE_FAILED);

	default:
		break;
	}

	if (taclog->debug & DEBUG_AUTHEN_FLAG)
		taclog->report("tauthen::choose_authen - Illegal packet action=%d type=%d from %s\n", action, type->authen_type, sock->getForeignAddress().c_str());
	return (CHOOSE_FAILED);
}

void tauthen::authenticate()
{
	int iterations = 0, ret = 0;
	u_char *reply, *p;
	struct authen_cont *cont;

	if (taclog->debug & DEBUG_AUTHEN_FLAG)
		taclog->report("tauthen::authenticate - Start authenticate\n");

	if (type->authen_func==0) {
		if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::authenticate - Cannot authenticate. No authentication function\n");
		sock->send_authen_error("Cannot authenticate");
		return;
	}

	while (1) {
		if (aborted) {
			return;
		}

		if (++iterations >= TAC_PLUS_MAX_ITERATIONS) {
			sock->send_authen_error("Too many iterations authenticating");
			return;
		}

		switch (type->authen_func) {
		case DEFAULT_V0_FN:
			ret = default_v0_fn();
			break;
		case DEFAULT_FN:
			ret = default_fn();
			break;
		case ENABLE_FN:
			ret = enable_fn();
			break;
		case SENDAUTH_FN:
			ret = sendauth_fn();
			break;
		case SENDPASS_FN:
			ret = sendpass_fn();
			break;
		}

		if (ret) {
			sock->send_authen_error("Authentication function failure");
			return;
		}

		switch (status) {
		default:
			sock->send_authen_error("Illegal status value returned from authentication function");
			return;

		case TAC_PLUS_AUTHEN_STATUS_PASS:
			access(); //Access logging - not vey efficent
			sock->send_authen_reply(TAC_PLUS_AUTHEN_STATUS_PASS, server_msg.c_str(), server_data.c_str(), 0);
			return;

		case TAC_PLUS_AUTHEN_STATUS_ERROR:
			access(); //Access logging - not vey efficent
			sock->send_authen_error(server_msg.c_str() ? server_msg.c_str() : "authentication function unspecified error");
			return;

		case TAC_PLUS_AUTHEN_STATUS_FAIL:
			access(); //Access logging - not vey efficent
			sock->send_authen_reply(TAC_PLUS_AUTHEN_STATUS_FAIL, server_msg.c_str(), server_data.c_str(), 0);
			return;

		case TAC_PLUS_AUTHEN_STATUS_GETUSER:
		case TAC_PLUS_AUTHEN_STATUS_GETPASS:
		case TAC_PLUS_AUTHEN_STATUS_GETDATA:
			sock->send_authen_reply(status, server_msg.c_str(), server_data.c_str(), flags);

			flags = 0;

//			if ((reply=sock->get_authen_cont())==NULL) {
			if (sock->get_authen_cont(&reply)) {
				if (taclog->debug & DEBUG_AUTHEN_FLAG)
					taclog->report("tauthen::authenticate - Null reply packet, expecting CONTINUE from %s port %u\n", sock->getForeignAddress().c_str(), sock->getForeignPort());
			
				//Tell authentication function it should clean up
				flags |= TAC_PLUS_CONTINUE_FLAG_ABORT;
//				flags = 0;
				return;
			}

			cont = (struct authen_cont *) (reply+TAC_PLUS_HDR_SIZE);
			if (cont->flags & TAC_PLUS_CONTINUE_FLAG_ABORT) {
				aborted = 1;
				flags |= TAC_PLUS_CONTINUE_FLAG_ABORT;
//				flags = 0;

				if (cont->user_data_len) {
					//Abort message exists. Create null string
					client_data = new char[cont->user_data_len+1];
					p = reply + TAC_PLUS_HDR_SIZE + TAC_AUTHEN_CONT_FIXED_FIELDS_SIZE + cont->user_msg_len;
					bcopy(p, client_data, cont->user_data_len);
					client_data[cont->user_data_len] = '\0';
				}
				delete reply;
				return;
			}

			p = reply + TAC_PLUS_HDR_SIZE + TAC_AUTHEN_CONT_FIXED_FIELDS_SIZE;

			switch (status) {

			case TAC_PLUS_AUTHEN_STATUS_GETDATA:
			case TAC_PLUS_AUTHEN_STATUS_GETPASS:
				client_msg = new char[cont->user_msg_len+1];
				bcopy(p, client_msg, cont->user_msg_len);
				client_msg[cont->user_msg_len] = '\0';
				delete reply;
				continue;

			case TAC_PLUS_AUTHEN_STATUS_GETUSER:
			default:
				sock->send_authen_error("Cannot authenticate");
				delete reply;
				return;
			}
		}
	}
}

int tauthen::default_v0_fn()
{
	int ret = 1;
	string prompt, *password;

	if ((service!=TAC_PLUS_AUTHEN_SVC_ENABLE) && id->username->empty()) {
		switch (state) {
		case STATE_AUTHEN_GETUSER:
			status = TAC_PLUS_AUTHEN_STATUS_ERROR;
			if (taclog->debug & DEBUG_AUTHEN_FLAG)
				taclog->report("tauthen::default_v0_fn - No user username supplied after GETUSER\n");
			return (0);
		case STATE_AUTHEN_START:
			status = TAC_PLUS_AUTHEN_STATUS_GETUSER;
			if (service == TAC_PLUS_AUTHEN_SVC_LOGIN) {
				dbh->get_host("prompt",id->NAS_addr, &prompt);
				server_msg.append(prompt+PROMPT_INSERT);
			} else
				server_msg.append("Username: ");
			state = STATE_AUTHEN_GETUSER;
			return (0);
		default:
			if (taclog->debug & DEBUG_AUTHEN_FLAG)
				taclog->report("tauthen::default_v0_fn - Bad state %d\n", state);
			return (1);
		}
	}

	if (!id->username->compare(DEFAULT_USERNAME)) {
		status = TAC_PLUS_AUTHEN_STATUS_FAIL;
		if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::default_v0_fn - authentication query for '%s' from %s on %s rejected\n", id->username->c_str(), sock->getForeignAddress().c_str(), id->NAS_port->c_str());
		return (0);
	}

	if (!password) {
		switch (state) {
			case STATE_AUTHEN_GETPASS:
				password = new string(client_msg);
				delete[] client_msg;
				break;

			default:
				flags = TAC_PLUS_AUTHEN_FLAG_NOECHO;
				server_msg.assign("Password: ");
				status = TAC_PLUS_AUTHEN_STATUS_GETPASS;
				state = STATE_AUTHEN_GETPASS;
				return (0);
		}
	}

	status = TAC_PLUS_AUTHEN_STATUS_ERROR;

	switch (service) {
	case TAC_PLUS_AUTHEN_SVC_NASI:
	case TAC_PLUS_AUTHEN_SVC_LOGIN:
	case TAC_PLUS_AUTHEN_SVC_PPP:
		ret = verify(password, S_login);
		if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::default_v0_fn - login query for '%s' %s on port %s from %s\n", id->username->c_str(), (status==TAC_PLUS_AUTHEN_STATUS_PASS)?"accepted":"rejected",id->NAS_port->c_str(), sock->getForeignAddress().c_str());
		server_msg.assign("");
		break;
	default:
		if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::default_v0_fn - bogus service (%d)\n", service); }

	delete password;

	return (ret);
}

int tauthen::default_fn()
{
	int ret = 1;
	string secret, password;

	if (id->username->empty()) {
		if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::default_fn - No user name to do verification\n");
		status = TAC_PLUS_AUTHEN_STATUS_ERROR;
		return (0);
	}

        if (!id->username->compare(DEFAULT_USERNAME)) {
		status = TAC_PLUS_AUTHEN_STATUS_FAIL;
		if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::default_fn - authentication query for '%s'from %s on %s rejected\n", id->username->c_str(), sock->getForeignAddress().c_str(), id->NAS_port->c_str());
		return (0);
	}

	status = TAC_PLUS_AUTHEN_STATUS_ERROR;

	switch (atype) {
	case TAC_PLUS_AUTHEN_TYPE_CHAP:
		u_char uid;
		int chal_len;

		uid = client_data[0];
		chal_len = client_dlen - 1 - MD5_LEN;
		if (chal_len > 0) {
			if (get_secret(&secret)) {
				if (taclog->debug & DEBUG_AUTHEN_FLAG)
					taclog->report("tauthen::default_fn - Cannot find CHAP secret\n");
				status = TAC_PLUS_AUTHEN_STATUS_FAIL;
			} else {
				char digest[MD5_LEN], *chal;
				u_char *mdp;
				int inlen;
				MD5_CTX mdcontext;

				inlen = sizeof(u_char)+secret.size()+chal_len;
				mdp = new u_char[inlen];
				mdp[0] = uid;
				bcopy(secret.c_str(),&mdp[1],secret.size());
				chal = client_data + 1;
				bcopy(chal, mdp+secret.size()+1, chal_len);
				MD5Init(&mdcontext);
				MD5Update(&mdcontext, mdp, inlen);
				MD5Final((u_char*) digest, &mdcontext);
				free(mdp);

				if (bcmp(digest, client_data+1+chal_len, MD5_LEN)) {
					status = TAC_PLUS_AUTHEN_STATUS_FAIL;
				} else {
					status = TAC_PLUS_AUTHEN_STATUS_PASS;
				}
			}
		}
		break;
#ifdef MSCHAP
	case TAC_PLUS_AUTHEN_TYPE_MSCHAP:
//MSCHAP Verification
		if (get_secret(&secret)) {
		} else {
		}
		break;
#endif
	case TAC_PLUS_AUTHEN_TYPE_ARAP:
		char nas_chal[8], r_chal[8], r_resp[8];
 
		bcopy(client_data, nas_chal, 8);
		bcopy(client_data+8, r_chal, 8);
		bcopy(client_data+16, r_resp, 8);

		if (get_secret(&secret)) {
			if (taclog->debug & DEBUG_AUTHEN_FLAG)
				taclog->report("tauthen::default_fn - Cannot find ARAP secret\n");
			status = TAC_PLUS_AUTHEN_STATUS_FAIL;
		} else {
			char *temp = new char[secret.size()+1],
			     *pws = new char[8];
			bcopy(secret.c_str(),temp,secret.size());
			bzero(pws, 8);
			for (int i=0; i<8 && temp[i]; i++)
				pws[i] = temp[i] << 1;

			secret.assign(pws);
			delete temp;
			delete pws;

#ifdef ARAP_DES
			des_init(0);
			des_setkey(secret.c_str());
			des_endes(nas_chal);
			des_done();
#endif

			if (bcmp(nas_chal, r_resp, 8)) {
				status = TAC_PLUS_AUTHEN_STATUS_FAIL;
			} else {
				status = TAC_PLUS_AUTHEN_STATUS_PASS;
			}

#ifdef ARAP_DES
			des_init(0);
			des_setkey(secret.c_str());
			des_endes(r_chal);
			des_done();
#endif
			server_data.append(r_chal);
		}
		break;
	case TAC_PLUS_AUTHEN_TYPE_PAP:
		password.assign(client_msg);
		ret=verify(&password, S_pap);
		delete[] client_msg;
		break;
	case  TAC_PLUS_AUTHEN_TYPE_ASCII:
		if (password.empty()) {
			switch (state) {
			case STATE_AUTHEN_GETPASS:
                                password.assign(client_msg);
				delete[] client_msg;
				break;

			case  STATE_AUTHEN_START:
				//NO Password feature is no longer good
				if (taclog->debug & DEBUG_AUTHEN_FLAG)
					taclog->report("tauthen::default_fn - feature for no password is disabled\n");
			default:
				flags = TAC_PLUS_AUTHEN_FLAG_NOECHO;
				server_msg.assign("Password: ");
				status = TAC_PLUS_AUTHEN_STATUS_GETPASS;
				state = STATE_AUTHEN_GETPASS;
				return (0);
			}
		}
		if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::default_fn - login query for '%s'\n", id->username->c_str());

		//We have username and password
		ret = verify(&password, S_login);
		break;
	default:
		break;
	}

	server_msg.assign("");
	return (ret);
}

int tauthen::enable_fn()
{
	int ret = 1;
	string *password;

	switch (state) {
	case STATE_AUTHEN_GETPASS:
		password = new string(client_msg);
		delete[] client_msg;
		break;

	default:
		flags = TAC_PLUS_AUTHEN_FLAG_NOECHO;
		server_msg.assign("Password: ");
		status = TAC_PLUS_AUTHEN_STATUS_GETPASS;
		state = STATE_AUTHEN_GETPASS;
		return (0);
	}

	switch (service) {
	case TAC_PLUS_AUTHEN_SVC_ENABLE:
		if ((id->priv_lvl<TAC_PLUS_PRIV_LVL_MIN)||(id->priv_lvl>TAC_PLUS_PRIV_LVL_MAX)) {
			status = TAC_PLUS_AUTHEN_STATUS_ERROR;
			server_msg.assign("Invalid privilege level");

			if (taclog->debug  & DEBUG_AUTHEN_FLAG)
				taclog->report("tauthen::enable_fn - privilege level %d is Invalid\n", id->priv_lvl);
			return (0);
		}

		ret = verify(password, S_enable);

		if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::enable_fn - Query for '%s' from %s on port %s %s\n", id->username->c_str(), sock->getForeignAddress().c_str(), id->NAS_port->c_str(), (status==TAC_PLUS_AUTHEN_STATUS_PASS)?"accepted":"rejected");
		if (status == TAC_PLUS_AUTHEN_STATUS_PASS) {
			server_msg.assign("\nEnable accepted.");
		} else {
			server_msg.assign("Enable rejected.");
		}
		break;

	default:
		status = TAC_PLUS_AUTHEN_STATUS_ERROR;
		if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::enable_fn - Bogus service %d\n", service);
		break;
	}

	delete password;

	return (ret);
}

int tauthen::sendauth_fn()
{
	int status, chal_len;
	string pmsg, exp_date, secret;
	char *chal, uid;

	status = TAC_PLUS_AUTHEN_STATUS_FAIL;

        if (!id->username->compare(DEFAULT_USERNAME))
	{
		if (id->username->empty()) {
			status = TAC_PLUS_AUTHEN_STATUS_ERROR;
			server_msg.assign("No username supplied");
			if (taclog->debug & DEBUG_AUTHEN_FLAG)
				taclog->report("tauthen::sendauth_fn - No username supplied\n");
			return (0);
		}

		switch (type->authen_type) {
		case TAC_PLUS_AUTHEN_TYPE_CHAP:
			uid = client_data[0];
			chal_len = client_dlen - 1;
			if (chal_len < 0) {
				status = TAC_PLUS_AUTHEN_STATUS_ERROR;
			} else {
				if (taclog->debug & DEBUG_AUTHEN_FLAG)
					taclog->report("tauthen::sendauth_fn - user (%s) uid (%d) chal_len (%d)\n", id->username->c_str(), (int)uid, chal_len);

				if (!get_secret(&secret)){
					if (!secret.empty()) {
						MD5_CTX mdcontext;
						u_char *mdp;
						char digest[MD5_LEN];
						int inlen = sizeof(u_char)+secret.size()+chal_len;

						mdp = new u_char[inlen];
						mdp[0] = uid;
						bcopy(secret.c_str(),&mdp[1],secret.size());
						chal = client_data + 1;
						bcopy(chal, mdp+secret.size()+1, chal_len);
						MD5Init(&mdcontext);
						MD5Update(&mdcontext, mdp, inlen);
						MD5Final((u_char*)digest, &mdcontext);
						delete mdp;

						//Now return the calculated response value
						server_msg.assign(digest);
						status = TAC_PLUS_AUTHEN_STATUS_PASS;
					} else {
						if (taclog->debug & DEBUG_AUTHEN_FLAG)
							taclog->report("tauthen::sendauth_fn - No chap or global secret can be found for %s\n", id->username->c_str());
					}
				}
			}
			break;
#ifdef MSCHAP
		case  TAC_PLUS_AUTHEN_TYPE_MSCHAP:
			if (client_dlen <= 2) {
				status = TAC_PLUS_AUTHEN_STATUS_ERROR;
			} else {
				uid = client_data[0];
				chal_len = client_dlen - 1;
				if (taclog->debug & DEBUG_AUTHEN_FLAG)
					taclog->report("tauthen::sendauth_fn - user (%s) uid (%d) chal_len (%d)\n",id->username.c_str(), (int)uid, chal_len);

				if (!get_secret(&secret)) {
					if (!secret.empty()) {
						char *server_d;
						chal = client_data + 1;
						server_d = new char[MSCHAP_DIGEST_LEN];
						mschap_lmchallengeresponse(chal,secret,&server_d[0]);
						mschap_ntchallengeresponse(chal,secret,&server_d[24]);

						server_d[48] = 1; /* Mark it to use the NT response*/
//						server_dlen = MSCHAP_DIGEST_LEN;
						server_data.assign(server_d);
						status = TAC_PLUS_AUTHEN_STATUS_PASS;
					} else 
						if (taclog->debug & DEBUG_AUTHEN_FLAG)
							taclog->report("tauthen::sendauth_fn - No mschap or global secret for %s\n", id->username);
				}
			}
			break;
#endif
		case TAC_PLUS_AUTHEN_TYPE_PAP:
			if (!get_secret(&secret)) {
				if (!secret.empty()) {
					server_data.assign(secret);
					status = TAC_PLUS_AUTHEN_STATUS_PASS;
				} else {
					if (taclog->debug & DEBUG_AUTHEN_FLAG)
						taclog->report("tauthen::sendauth_fn - No PAP secret\n");
				}
			}
			break;

		default:
			status = TAC_PLUS_AUTHEN_STATUS_ERROR;
			if (taclog->debug & DEBUG_AUTHEN_FLAG)
				taclog->report("tauthen::sendauth_fn - Illegal data type\n");
		}
	}

	return (0);
}

int tauthen::sendpass_fn()
{
	int status = 0, expired;

	status = TAC_PLUS_AUTHEN_STATUS_FAIL;

	if (taccfg->sendauth_only) {
		if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::sendpass_fn - Sendpass request rejected\n", id->username->c_str());
		return (status);
	}

	if (id->username->empty()) {
		status = TAC_PLUS_AUTHEN_STATUS_ERROR;
		server_msg.assign("No username supplied");
		if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::sendpass_fn - No user supplied\n");
		return (status);
	}

        if (!id->username->compare(DEFAULT_USERNAME)) {
		string secret;

		if (!get_secret(&secret)) {
			if (secret.empty()) {
				server_data.assign("No secret");
			} else {
				server_data.assign(secret);
			}
		}
	}

	if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::sendpass_fn - Sendpass query for '%s' %s\n", id->username->c_str(), (status == TAC_PLUS_AUTHEN_STATUS_PASS)?"accepted":"rejected");

	return (status);
	
}

int tauthen::get_secret(string *secret)
{
	int ret=0,
	    expired=PW_EXPIRED; //Assume the worst case
	time_t t,
	       now = time(NULL);

	switch (taccfg->cfgtype) {
	case USEFILE:
		secret->assign("secret");
		break;

	case USEDB:
		if (dbh->get_secret(*id->username, atype, secret, t)) {
			expired = 3;
		} else {
			if ((t-now)<=WARNING_PERIOD) expired = PW_EXPIRING;
			else if (t>now) expired = PW_OK;
		}
		break;

	default:
		if (taclog->debug & DEBUG_PASSWD_FLAG)
			taclog->report("tauthen::get_secret - Cannot find type\n");
	}
		
	switch(expired) {
	case PW_OK:
		if (taclog->debug & DEBUG_PASSWD_FLAG)
			taclog->report("tauthen::get_secret - Password has not expired\n");
		status = TAC_PLUS_AUTHEN_STATUS_PASS;
		server_msg.assign("Signon successful.");
		break;

	case  PW_EXPIRING:
		if (taclog->debug & DEBUG_PASSWD_FLAG)
			taclog->report("tauthen::get_secret - Password will expire soon\n");
		status = TAC_PLUS_AUTHEN_STATUS_PASS;
		server_msg.assign("Password will expire soon.");
		break;

	case PW_EXPIRED:
		if (taclog->debug & DEBUG_PASSWD_FLAG)
			taclog->report("tauthen::get_secret - Password has expired\n");
		status = TAC_PLUS_AUTHEN_STATUS_FAIL;
		server_msg.assign("Password has expired.");
		ret = 1;
		break;

	default:
		if (taclog->debug & DEBUG_PASSWD_FLAG)
			taclog->report("tauthen::get_secret - Bogus return value\n");
		status = TAC_PLUS_AUTHEN_STATUS_ERROR;
		ret = 2;
		break;
	}

	return (ret) ;
}

int tauthen::verify(string *password, int auth_type) {
	int ret = 0;

	taclog->report("tauthen::verify - Verification\n");

	if (id->username->empty()) {
		taclog->report("tauthen::verify - Empty username\n");
		status = TAC_PLUS_AUTHEN_STATUS_ERROR;
		return (1);
	}

	status = TAC_PLUS_AUTHEN_STATUS_FAIL;

	switch (taccfg->cfgtype) {
	case USEFILE:
		status = TAC_PLUS_AUTHEN_STATUS_FAIL;
		break;

	case USEDB:
		status = dbh->verify((char*)password->c_str(), id, auth_type);
		break;

	default:
		if (taclog->debug & DEBUG_AUTHEN_FLAG)
			taclog->report("tauthen::verify - No such configuration type\n");
		ret = 1;
		status = TAC_PLUS_AUTHEN_STATUS_ERROR;
	}

	return (ret);
}

void tauthen::access()
{
	string querycmd(ACCESSSQL2);
        string comma("','");
        int ret = 0;
                                                                                
        DB_RES *res;
        DB_ROW row;
                                                                                
	if (taclog->debug)
        	taclog->report("tauthen::access - Logging user access\n");
                                                                                
        querycmd += *id->username;
        querycmd += comma+*id->NAS_name;
        querycmd += comma+*id->NAC_address;
        querycmd += comma+*id->NAS_port;
        querycmd += (status == TAC_PLUS_AUTHEN_STATUS_PASS) ? string("','accepted'") : string("','rejected'");

        switch (service) {
                case TAC_PLUS_AUTHEN_SVC_LOGIN:
                        querycmd += ",'login'";
                        break;
                case TAC_PLUS_AUTHEN_SVC_ENABLE:
                        querycmd += ",'enable'";
                        break;
                case TAC_PLUS_AUTHEN_SVC_PPP:
                        querycmd += ",'ppp'";
                        break;
                case TAC_PLUS_AUTHEN_SVC_ARAP:
                        querycmd += ",'arap'";
                        break;
                case TAC_PLUS_AUTHEN_SVC_PT:
                        querycmd += ",'pt'";
                        break;
                case TAC_PLUS_AUTHEN_SVC_RCMD:
                        querycmd += ",'rcmd'";
                        break;
                case TAC_PLUS_AUTHEN_SVC_X25:
                        querycmd += ",'x25'";
                        break;
                case TAC_PLUS_AUTHEN_SVC_NASI:
                        querycmd += ",'nasi'";
                        break;
                default:
                        querycmd += ",'unknown'";
        }
                                                                                
        querycmd += ")";

	dbh->acct(&querycmd);
}
