/*
 *   Copyright (C) 2002-2006
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <netinet/in.h>
#include "tac_plus.h"
#include "tacacs.h"
#include "md5.h"

tauthor::tauthor(tacacs *session) {
	sock = session->sock;
	dbh = session->dbh;
	fileh = session->fileh;

	aborted = 0;

	status = AUTHOR_STATUS_FAIL;		//Assume the worst case
	admin_msg = new string();
	msg = new string();
}

tauthor::~tauthor() {
	if (admin_msg != NULL) delete admin_msg;
	if (msg != NULL) delete msg;
}

int tauthor::authorize()
{
	if (taclog->debug & DEBUG_AUTHOR_FLAG) {
		taclog->report("tauthor::authorize - authorizing user(%s) on nas(%s)\n", id->username->c_str(), sock->getForeignAddress().c_str());
	}

	switch (taccfg->cfgtype) {
	case USEFILE:
		break;

	case  USEDB:
		dbh->author(this);
		break;

	default:
		taclog->report("tauthor::authorize - No such configuration type\n");
		status = AUTHOR_STATUS_ERROR;
	}

        sock->send_author_reply(status, msg, admin_msg, output_args);

	return (status);
}

int tauthor::author_svc(map<string,string> cfg_args, int denybydefault)
{
	int replaced = 0,
	    added = 0;
	string cfg_arg;
	map<string,string>::const_iterator iter;

	for (iter=input_args.begin(); iter!=input_args.end(); ++iter)
	{
		if ((iter->first.find("service")==0) ||
		    (iter->first.find("protocol")==0) ||
		    (iter->first.find("cmd")==0)) {
			if (taclog->debug & DEBUG_AUTHOR_FLAG)
				taclog->report("tauthor::author_svc - argument pass through (%s%s)\n",iter->first.c_str(), iter->second.c_str());
			output_args[iter->first]=iter->second;
			continue;
		}

		if (iter->second.find("=")==0) {
			/* a) look for an exact attribute,value match in the daemon's mandatory list.  If found, add the AV pair to the output */

			if (cfg_args[iter->first] == iter->second) {
				if (taclog->debug & DEBUG_AUTHOR_FLAG)
					taclog->report("tauthor::author_svc -a- nas AV pair(%s%s) config AV pair(%s%s) -> add nas AV pair\n", iter->first.c_str(), iter->second.c_str(), iter->first.c_str(), cfg_args[iter->first].c_str());

				output_args[iter->first]=iter->second;
				continue;
			}

			/* b) if an exact match doesn't exist, look in the daemon's optional list for the first attribute match.  If found, add the NAS AV pair to the output. */

			if (!cfg_args[iter->first].empty()) {
				if (taclog->debug & DEBUG_AUTHOR_FLAG)
					taclog->report("tauthor::author_svc -b- nas AV pair(%s%s) config AV pair(%s%s) -> add nas AV pair\n", iter->first.c_str(), iter->second.c_str(), iter->first.c_str(), cfg_args[iter->first].c_str());
				output_args[iter->first]=iter->second;
				continue;
			}

			/* c) if no attributes match exists, deny the command if the default is to deny */

			if (denybydefault) {
				status = AUTHOR_STATUS_FAIL;

				if (taclog->debug & DEBUG_AUTHOR_FLAG)
					taclog->report("tauthor::author_svc -c- nas AV pair (%s%s) config AV pair (absent), default deny\n", iter->first.c_str(), iter->second.c_str());

				output_args.clear();
				cfg_args.clear();
				return (0);
			}

			/* d) if default is permit, add the nas AV pair to the output */
			if (taclog->debug & DEBUG_AUTHOR_FLAG)
				taclog->report("tauthor::author_svc -d- nas AV pair(%s%s) config AV pair(absent), default permit -> add nas AV pair\n", iter->first.c_str(), iter->second.c_str());

			output_args[iter->first]=iter->second;
		} else {
			/* nas AV pair optional */
			cfg_arg = cfg_args[iter->first];

			if (!cfg_arg.empty()) {
			/* e) look for exact attribute,value match in the mandatory list. If found, add config AV pair to output. */

				if (cfg_arg.find("=")==0) {
					if (iter->second.substr(1) == cfg_arg.substr(1)) {
						if (taclog->debug & DEBUG_AUTHOR_FLAG)
							taclog->report("tauthor::author_svc -e- nas AV pair(%s%s) config AV pair(%s%s) -> replace with config AV pair\n", iter->first.c_str(), iter->second.c_str(), iter->first.c_str(), cfg_arg.c_str());

						output_args[iter->first]=cfg_arg;
						replaced++;
						continue;
					}

			/* f) If not found, look for the first attribute match in the mandatory list.  If found, add config AV pair to output. */

					if (taclog-> debug & DEBUG_AUTHOR_FLAG)
						taclog->report("tauthor::author_svc - f- nas AV pair(%s%s) config AV pair(%s%s) -> replace with config AV pair\n", iter->first.c_str(), iter->second.c_str(), iter->first.c_str(), cfg_arg.c_str());

					output_args[iter->first]=cfg_arg;
					replaced++;
					continue;
				}

			/* g) If no mandatory match exists, look for an exact attribute,value pair match among the config optional AV pairs. If found, add config AV pair to output. */

				if (cfg_arg.find("*")==0) {
					if (iter->second.substr(1)==cfg_arg.substr(1)) {
						if (taclog->debug & DEBUG_AUTHOR_FLAG)
							taclog->report("tauthor::author_svc -g- nas AV pair(%s%s) config AV pair(%s%s) -> replace with config AV pair\n", iter->first.c_str(), iter->second.c_str(), iter->first.c_str(), cfg_arg.c_str());

						output_args[iter->first]=cfg_arg;
						replaced++;
						continue;
					}

			/* h) If no exact match exists, locate the first attribute match among the config optional AV pairs. If found, add config AV pair to output. */
					if (taclog->debug & DEBUG_AUTHOR_FLAG)
						taclog->report("tauthor::author_svc -h- nas AV pair(%s%s) config AV pair(%s%s) -> replace with config AV pair\n", iter->first.c_str(), iter->second.c_str(), iter->first.c_str(), cfg_arg.c_str());
					output_args[iter->first]=cfg_arg;
					replaced++;
					continue;
				}
			}

			/* i) no match found. delete the AV pair if default is deny */

			if (denybydefault) {
				if (taclog->debug & DEBUG_AUTHOR_FLAG)
					taclog->report("tauthor::author_svc -i- nas AV pair(%s%s) config AV pair(absent) deny -> delete nas AV pair\n", iter->first.c_str(), iter->second.c_str());

				replaced++;
				goto next_nas_arg;
			}

			/* j) no match. defaul tis to permit. */

			if (taclog->debug & DEBUG_AUTHOR_FLAG)
				taclog->report("tauthor::author_svc -j- nas AV pair(%s%s) config AV pair(absent) permit -> add nas AV pair\n");
			output_args[iter->first]=iter->second;
		}

	next_nas_arg:;
	}

	/* k) After all AV pairs have been processed, for each mandatory configuration pair and if there is no attribute match already in output list, add the AV pair (add only one AV pair for each mandatory attribute) */

	for (iter=cfg_args.begin();iter!=cfg_args.end();++iter) {
		if (iter->second.find("*")==0) continue;
		if (output_args[iter->first] == iter->second) continue;

		if (taclog->debug & DEBUG_AUTHOR_FLAG)
			taclog->report("tauthor::author_svc -k- NAS AV pair absent config AV pair(%s%s) -> added config\n",iter->first.c_str(),iter->second.c_str());

		added++;
		output_args[iter->first]=iter->second;
	}

	if (replaced) {
		if (taclog->debug & DEBUG_AUTHOR_FLAG)
			taclog->report("tauthor::author_svc - replaced %d args\n", replaced);

		status = AUTHOR_STATUS_PASS_REPL;
		cfg_args.clear();

		return (0);
	}

	if (added) {
		if (taclog->debug & DEBUG_AUTHOR_FLAG)
			taclog->report("tauthor::author_svc - added %d args\n", added);

		/** cleanup the output args and remove copies of input args **/
		for (iter=input_args.begin();iter!=input_args.end();++iter){
			if (output_args[iter->first]==iter->second) {
				output_args.erase(iter->first);
			}
		}

		status = AUTHOR_STATUS_PASS_ADD;
		cfg_args.clear();
		return (0);
	}

	if (taclog->debug & DEBUG_AUTHOR_FLAG)
		taclog->report("tauthor::author_svc - nothing added or replaced\n");
	status = AUTHOR_STATUS_PASS_ADD;
	cfg_args.clear();
	output_args.clear();

	return (0);
}

int tauthor::pre_authorization(string cmd) {
	if (taclog->debug & DEBUG_AUTHOR_FLAG)
		taclog->report("tauthor::pre_authorization -- feature not implemented\n");

	return (0);
}

void tauthor::post_authorization(string cmd) {
	if (taclog->debug & DEBUG_AUTHOR_FLAG)
		taclog->report("tauthor::post_authorization -- feature not implemented\n");

	return;
}

int tauthor::get_nas_svc( string *cmd, string *protocol, string *svc)
{
	map<string,string>::const_iterator iter;

	iter = input_args.find("service");

	if (iter!=input_args.end()) {
		if (iter->second=="=shell") {
			iter = input_args.find("cmd");
			if (iter!=input_args.end()) {
				if (iter->second.size()<=1)
					return (N_svc_exec);
	
				cmd->assign(iter->second.substr(1));
				return (N_svc_cmd);
			}

			return (0);
		}

		if (iter->second=="=slip")
			return (N_svc_slip);

		if (iter->second=="=arap")
			return (N_svc_arap);

		if (iter->second=="=ppp") {
			protocol->assign(input_args["protocol"].substr(1));
			return (N_svc_ppp);
		}

		svc->assign(iter->second.substr(1));
		return (N_svc);
	}

	return (0);
}
