/*
 *   Copyright (C) 2002-2006
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef DB_H
#define DB_H 1

#define USEDB 1

#if defined(DB_MYSQL)
#include <mysql.h>
typedef MYSQL DBH;
typedef MYSQL_ROW DB_ROW;
typedef MYSQL_RES DB_RES;
#define DB_TYPE 1
#endif

#if defined(DB_PGSQL)
#include <libpq-fe.h>
#define PWLEN   13
typedef PGresult DB_RES;
typedef struct {
        Gconn *conn;
        DB_RES *res;
} DBH;
#define DB_TYPE 2
#endif

#if defined(DB_ODBC)
#include <odbc.h>
#define DB_TYPE 3
#endif

#if defined(DB_ORACLE)
#include <oracle.h>
#define DB_TYPE 4
#endif

#define SQLCMDL 2048

#define AUTHSQL "SELECT %s, gid, disable, auth FROM user WHERE uid='%s' and user=1 and (UNIX_TIMESTAMP(expires)>UNIX_TIMESTAMP() or expires='0000-00-00 00:00:00')"
#define NEWAUTHSQL "SELECT %s, gid, disable, auth, UNIX_TIMESTAMP(expires) FROM user WHERE uid='%s' and user=1"
#define HOSTAUTHSQL "SELECT %s, hostgroup FROM host WHERE network=(%u&submask) and host=1 ORDER BY submask DESC"
#define USERSECRETSQL "SELECT %s, UNIX_TIMESTAMP(expires) FROM user WHERE uid='%s' and user=1"
#define HOSTGROUPSQL "SELECT %s FROM host WHERE ip='%s'"
#define USERACCESSSQL "SELECT permission FROM user, acl where acl_id=acl.id and uid in ('%s', '%s') and type=1 and value1=(INET_ATON('%s')&submask)"
#define HOSTACLSQL "SELECT %s, hostgroup, vendor FROM host WHERE network=(%u&submask) and host=1 ORDER BY submask DESC"
#define NASACLSQL "SELECT permission, value1 FROM acl WHERE id=%d AND value IN ('%s', '%s','allusers') AND type=2 ORDER BY seq"

#define ACCTSQL "INSERT INTO accounting (date,nas,uid,terminal,client_ip,type,service,priv_lvl,elapsed_time,bytes_in,bytes_out,cmd) VALUES (NOW(),'%s','%s','%s','%s','%s','%s',%s,%s,%s,%s,'%s')"
//#define ACCTSQL2 "INSERT INTO accounting (date,nas,uid,terminal,client_ip,type,service,priv_lvl,elapsed_time,bytes_in,bytes_out,cmd) VALUES (NOW(),'"
#define ACCTSQL2 "INSERT INTO accounting set date=NOW(), "
#define ACCESSSQL "INSERT INTO access (date,nas,terminal,uid,client_ip,service,status) VALUES (NOW(),'%s','%s','%s','%s','%s','%s')"
#define ACCESSSQL2 "INSERT INTO access (date,uid,nas,client_ip,terminal,status,service) VALUES (NOW(),'"

#define DEFAULT_DB_TABLE_NAME "user"
#define DEFAULT_DB_FIELD_NAME_USER   "uid"
#define DEFAULT_DB_FIELD_NAME_PASSWD "password"

#define DEFAULT_DB "mysql://tacacs:tac_plus@localhost/tacacs"

struct author_user {
	int svc_dflt;
	int cmd_dflt;
	string username;
	string member;
	string pre_author;
	string post_author;
	uint32_t profile;
	int vid;
};

class db {
	int connected;  // state of the connection

	string type;
	string user;
	string passwd;
	string host;
	string database;

	DBH *dbh;

	int get_profile(struct identity *id, string member, int auth_type, uint32_t *profile, int *vendor);
	int get_svc_node(string user, int vendor, int svc, string value, string value1);
	int authorize_cmd(struct author_user *user, string cmd, tauthor *data);
	int authorize_exec(struct author_user *user, tauthor *data);
	int authorize_svc(struct author_user *user, int svc, string protocol, string svcname, tauthor *data);

   public:
	db(string *url);
	db(const db&);
	~db();
	int connect();
	void close();
	DB_RES *store_result();
	int num_rows( DB_RES *result );
	DB_ROW fetch_row( DB_RES *result );
	void free_result( DB_RES *result );
	int affected_rows();
	int query(string *querystmt);
#ifdef __STDC__
	int query(char *querystmt,...);
#else
	int query(querystmt, va_alist);
#endif

	int setcfg(string *url);
	int get_secret(string user, int type, string*secret, time_t time);
	int get_host(char *field, uint32_t nas, string *data);
	int get_user(struct identity *id, struct author_user *user);
	int acct(string *querystr);
	int verify(char *user_passwd, struct identity *id, int auth_type);

	int author(tauthor *data);

};
#endif
