/*
 *   Copyright (C) 2002-2006
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include <fstream>
#include <stdarg.h>
#include "tac_plus.h"

file::file(string *afile) {
	acctfile = new string(afile->c_str());
}

file::~file(){
	delete acctfile;
}

void file::init() {
	if (taclog->debug)
		taclog->report("TACACS accounting file started\n");
}

int file::acct(struct acct_rec *rec, map<string,string>& args)
{
	int ret = 0;
	fstream log;

	log.open(acctfile->c_str(), fstream::out | fstream::app);
	if (log.is_open()) {
		time_t t = time(NULL);
		log << string(ctime(&t));
		log << "\t" << rec->identity->NAS_name;
		log << "\t" << rec->identity->username;
		log << "\t" << rec->identity->NAS_port;
		log << "\t" << rec->identity->NAC_address;

		switch (rec->acct_type) {
		case ACCT_TYPE_UPDATE:
			log << "\tupdate";
			break;
		case ACCT_TYPE_START:
			log << "\tstart";
			break;
		case ACCT_TYPE_STOP:
			log << "\tstop";
			break;
		default:
			log << "\tunknown";
		}

		map<string,string>::const_iterator iter;
		for (iter=args.begin(); iter!=args.end(); ++iter) {
			log << "\t" << iter->first <<"="<<iter->second << "\n";
		}
/*
		for (int i=0; i < rec->num_args; i++)
			log << "\t" << rec->args[i];
*/

		log << endl;
		log.close();
	} else {
		if (taclog->debug)
			taclog->report("Cannot open accounting file %s\n", acctfile);
		ret = 1;
	}

	return (ret);
}

