/* acexport.h -- required include file for ACE/Agent API calls */
/******************************************************************************
*               COPYRIGHT (C) 1995-2000 by RSA Security Inc.                  *
*                         ---ALL RIGHTS RESERVED---                           *
******************************************************************************/

#ifndef ACEXPORT_H
#define ACEXPORT_H

#include <time.h>

#ifdef WIN32
#pragma pack(push,8)
#endif

#ifdef  __cplusplus
extern "C" {
#endif

#include "sdacmvls.h"       /* return codes from ACE/Server */
#include "acclnt.h"         /* return codes from AceXxx functions */
#include "sdi_defs.h"       /* definition of SD_CLIENT structure */
#include "sd_types.h"       /* new portability macros */

/* Holds pin limits and related values */
typedef struct tagSD_PIN
{
	SD_CHAR Min;
    SD_CHAR Max;
    SD_CHAR Selectable;
    SD_CHAR Alphanumeric;
	SD_CHAR System[LENPRNST+2];
} SD_PIN;

/* ACE/Agent API handle */
typedef int SDI_HANDLE, * LP_SDI_HANDLE;

/* invalid ACE/Agent API handle */
#define SDI_HANDLE_NONE 0

#ifndef WIN32 /* Unix will use these prototypes */

typedef void (*ACECALLBACK) (SDI_HANDLE);

/* initialization and shutdown calls */
SD_BOOL AceInitialize   ( void );
SD_BOOL AceInitializeEx ( SD_CHAR *pszConfigFile, SD_CHAR *pszSharedMemoryName, SD_U32 u32HashIndexSize );
SD_BOOL AceShutdown     ( ACECALLBACK CleanupCallback );
void    AceCleanup      ( ACECALLBACK CleanupCallback );

/* New Asynchronous and thread-safe calls */
int  AceInit (LP_SDI_HANDLE pSdiHandle, void * userdata, ACECALLBACK appCallback);
int  AceLock (SDI_HANDLE SdiHandle, ACECALLBACK appCallback);
int  AceCheck (SDI_HANDLE SdiHandle, ACECALLBACK appCallback);
int  AceClientCheck (SDI_HANDLE SdiHandle, ACECALLBACK appCallback);
int  AceSendNextPasscode (SDI_HANDLE SdiHandle, ACECALLBACK appCallback);
int  AceSendPin (SDI_HANDLE SdiHandle, ACECALLBACK appCallback);
int  AceCancelPin (SDI_HANDLE SdiHandle, ACECALLBACK appCallback);
int  AceClose (SDI_HANDLE SdiHandle, ACECALLBACK appCallback);
int  AceGetAuthenticationStatus (SDI_HANDLE SdiHandle, SD_I32 *val);
int  AceGetMinPinLen (SDI_HANDLE SdiHandle, SD_CHAR *val);
int  AceGetMaxPinLen (SDI_HANDLE SdiHandle, SD_CHAR *val);
int  AceGetUserSelectable (SDI_HANDLE SdiHandle, SD_CHAR *val);
int  AceGetAlphanumeric (SDI_HANDLE SdiHandle, SD_CHAR *val);
int  AceGetSystemPin (SDI_HANDLE SdiHandle, SD_CHAR *val);
int  AceGetPinParams (SDI_HANDLE SdiHandle, SD_PIN *val);
int  AceGetUserData  (SDI_HANDLE SdiHandle, void **val);
int  AceGetTime (SDI_HANDLE SdiHandle, SD_I32 *val);
int  AceGetShell (SDI_HANDLE SdiHandle, SD_CHAR *val);
int  AceSetPasscode (SDI_HANDLE SdiHandle, SD_CHAR *val);
int  AceSetUsername (SDI_HANDLE SdiHandle, SD_CHAR *val);
int  AceSetNextPasscode (SDI_HANDLE SdiHandle, SD_CHAR *val);
int  AceSetPin (SDI_HANDLE SdiHandle, SD_CHAR *val);
int  AceSetUserClientAddress (SDI_HANDLE SdiHandle, SD_UCHAR *val);
int  AceSetUserData (SDI_HANDLE SdiHandle, void * val);
int  AceSetTimeout (SDI_HANDLE hdl, time_t lifetime, ACECALLBACK CleanupCallback);

/* New Synchronous and thread-safe calls */
int  SD_Init ( LP_SDI_HANDLE );
int  SD_Lock ( SDI_HANDLE, SD_CHAR * );
int  SD_Check ( SDI_HANDLE, SD_CHAR *, SD_CHAR * );
int  SD_ClientCheck ( SDI_HANDLE, SD_CHAR *, SD_CHAR *, SD_U32 );
int  SD_Next ( SDI_HANDLE, SD_CHAR * );
int  SD_Pin ( SDI_HANDLE, SD_CHAR * );
int  SD_Close ( SDI_HANDLE );

/* New Synchronous and thread-safe calls */
SD_ERROR AceStartAuth(LP_SDI_HANDLE,
    SD_CHAR *userID,         
    SD_I32 userIDLen,     
    SD_BOOL *moreData,    
    SD_BOOL *noEcho,      
    SD_I32 *respTimeout,  
    SD_I32 *nextRespLen,  
    SD_CHAR *promptStr,      
    SD_I32 *promptStrLen);

SD_ERROR AceContinueAuth(SDI_HANDLE, 
    SD_CHAR *userID,         
    SD_I32 userIDLen,     
    SD_BOOL *moreData,    
    SD_BOOL *noEcho,      
    SD_I32 *respTimeout,  
    SD_I32 *nextRespLen,  
    SD_CHAR *promptStr,      
    SD_I32 *promptStrLen);

SD_ERROR AceCloseAuth(SDI_HANDLE);

#else   /* Not Unix, WIN32-NT */

typedef void (WINAPI *ACECALLBACK)(SDI_HANDLE);

#ifndef USE_ACE_AGENT_API_PROTOTYPES

/* initialization and shutdown calls */
typedef SD_BOOL (WINAPI *ACEINITIALIZEPROC)   ( void );
typedef SD_BOOL (WINAPI *ACEINITIALIZEEXPROC) ( SD_CHAR *, SD_CHAR *, SD_U32 );
typedef SD_BOOL (WINAPI *ACESHUTDOWNPROC)     ( ACECALLBACK );
typedef void    (WINAPI *ACECLEANUPPROC)      ( ACECALLBACK );

/* New Asynchronous and thread-safe calls */
typedef int  (WINAPI *ACEINITPROC)(LP_SDI_HANDLE, void *, ACECALLBACK);
typedef int  (WINAPI *ACELOCKPROC)(SDI_HANDLE, ACECALLBACK);
typedef int  (WINAPI *ACECHECKPROC)(SDI_HANDLE, ACECALLBACK);
typedef int  (WINAPI *ACECLIENTCHECKPROC)(SDI_HANDLE, ACECALLBACK);
typedef int  (WINAPI *ACESENDNEXTPASSCODEPROC)(SDI_HANDLE, ACECALLBACK);
typedef int  (WINAPI *ACESENDPINPROC)(SDI_HANDLE, ACECALLBACK);
typedef int  (WINAPI *ACECANCELPINPROC)(SDI_HANDLE,  ACECALLBACK);
typedef int  (WINAPI *ACECLOSEPROC)(SDI_HANDLE, ACECALLBACK);
typedef int  (WINAPI *ACEGETAUTHENTICATIONSTATUSPROC)(SDI_HANDLE, SD_I32 *);
typedef int  (WINAPI *ACEGETMINPINLENPROC)(SDI_HANDLE, SD_CHAR *);
typedef int  (WINAPI *ACEGETMAXPINLENPROC)(SDI_HANDLE, SD_CHAR *);
typedef int  (WINAPI *ACEGETUSERSELECTABLEPROC)(SDI_HANDLE, SD_CHAR *);
typedef int  (WINAPI *ACEGETALPHANUMERICPROC)(SDI_HANDLE, SD_CHAR *);
typedef int  (WINAPI *ACEGETSYSTEMPINPROC)(SDI_HANDLE, SD_CHAR *);
typedef int  (WINAPI *ACEGETPINPARAMSPROC)(SDI_HANDLE, SD_PIN *);
typedef int  (WINAPI *ACEGETUSERDATAPROC)(SDI_HANDLE, void **);
typedef int  (WINAPI *ACEGETTIMEPROC)(SDI_HANDLE, SD_I32 *);
typedef int  (WINAPI *ACEGETSHELLPROC)(SDI_HANDLE, SD_CHAR *);
typedef int  (WINAPI *ACESETPASSCODEPROC)(SDI_HANDLE, SD_CHAR *);
typedef int  (WINAPI *ACESETUSERNAMEPROC)(SDI_HANDLE, SD_CHAR *);
typedef int  (WINAPI *ACESETNEXTPASSCODEPROC)(SDI_HANDLE, SD_CHAR *);
typedef int  (WINAPI *ACESETPINPROC)(SDI_HANDLE, SD_CHAR *);
typedef int  (WINAPI *ACESETUSERCLIENTADDRESSPROC)(SDI_HANDLE, SD_UCHAR *);
typedef int  (WINAPI *ACESETUSERDATAPROC)(SDI_HANDLE, void *);
typedef int  (WINAPI *ACESETTIMEOUTPROC)(SDI_HANDLE, time_t, ACECALLBACK);

/* New Synchronous and thread-safe calls */
typedef int  (WINAPI *NEWSDINITPROC)        ( LP_SDI_HANDLE );
typedef int  (WINAPI *NEWSDLOCKPROC)        ( SDI_HANDLE, SD_CHAR * );
typedef int  (WINAPI *NEWSDCHECKPROC)       ( SDI_HANDLE, SD_CHAR *, SD_CHAR * );
typedef int  (WINAPI *NEWSDCLIENTCHECKPROC) ( SDI_HANDLE, SD_CHAR *, SD_CHAR *, SD_U32 );
typedef int  (WINAPI *NEWSDNEXTPROC)        ( SDI_HANDLE, SD_CHAR * );
typedef int  (WINAPI *NEWSDPINPROC)         ( SDI_HANDLE, SD_CHAR * );
typedef int  (WINAPI *NEWSDCLOSEPROC)       ( SDI_HANDLE );

/* New Synchronous and thread-safe calls */
typedef SD_ERROR (WINAPI *ACESTARTAUTHPROC)(LP_SDI_HANDLE,
    SD_CHAR *userID,         
    SD_I32 userIDLen,     
    SD_BOOL *moreData,    
    SD_BOOL *noEcho,      
    SD_I32 *respTimeout,  
    SD_I32 *nextRespLen,  
    SD_CHAR *promptStr,      
    SD_I32 *promptStrLen);

typedef SD_ERROR (WINAPI *ACECONTINUEAUTHPROC)(SDI_HANDLE, 
    SD_CHAR *userID,         
    SD_I32 userIDLen,     
    SD_BOOL *moreData,    
    SD_BOOL *noEcho,      
    SD_I32 *respTimeout,  
    SD_I32 *nextRespLen,  
    SD_CHAR *promptStr,      
    SD_I32 *promptStrLen);

typedef SD_ERROR (WINAPI *ACECLOSEAUTHPROC)(SDI_HANDLE);

#else  /* !USE_ACE_AGENT_API_PROTOTYPES */

/* initialization and shutdown calls */
SD_BOOL WINAPI AceInitialize   ( void );
SD_BOOL WINAPI AceInitializeEx ( SD_CHAR *pszConfigFile, SD_CHAR *pszSharedMemoryName, SD_U32 u32HashIndexSize );
SD_BOOL WINAPI AceShutdown     ( ACECALLBACK CleanupCallback );
void    WINAPI AceCleanup      ( ACECALLBACK CleanupCallback );

/* New Asynchronous and thread-safe calls */
int  WINAPI AceInit(LP_SDI_HANDLE, void *, ACECALLBACK);
int  WINAPI AceLock(SDI_HANDLE, ACECALLBACK);
int  WINAPI AceCheck(SDI_HANDLE, ACECALLBACK);
int  WINAPI AceClientCheck(SDI_HANDLE, ACECALLBACK);
int  WINAPI AceSendNextPasscode(SDI_HANDLE, ACECALLBACK);
int  WINAPI AceSendPin(SDI_HANDLE, ACECALLBACK);
int  WINAPI AceCancelPin(SDI_HANDLE,  ACECALLBACK);
int  WINAPI AceClose(SDI_HANDLE, ACECALLBACK);
int  WINAPI AceGetAuthenticationStatus(SDI_HANDLE, SD_I32 *);
int  WINAPI AceGetMinPinLen(SDI_HANDLE, SD_CHAR *);
int  WINAPI AceGetMaxPinLen(SDI_HANDLE, SD_CHAR *);
int  WINAPI AceGetUserSelectable(SDI_HANDLE, SD_CHAR *);
int  WINAPI AceGetAlphanumeric(SDI_HANDLE, SD_CHAR *);
int  WINAPI AceGetSystemPin(SDI_HANDLE, SD_CHAR *);
int  WINAPI AceGetPinParams(SDI_HANDLE, SD_PIN*);
int  WINAPI AceGetUserData(SDI_HANDLE, void **);
int  WINAPI AceGetTime(SDI_HANDLE, SD_I32 *);
int  WINAPI AceGetShell(SDI_HANDLE, SD_CHAR *);
int  WINAPI AceSetPasscode(SDI_HANDLE, SD_CHAR *);
int  WINAPI AceSetUsername(SDI_HANDLE, SD_CHAR *);
int  WINAPI AceSetNextPasscode(SDI_HANDLE, SD_CHAR *);
int  WINAPI AceSetPin(SDI_HANDLE, SD_CHAR *);
int  WINAPI AceSetUserClientAddress(SDI_HANDLE, SD_UCHAR *);
int  WINAPI AceSetUserData(SDI_HANDLE, void *);
int  WINAPI AceSetTimeout(SDI_HANDLE, time_t, ACECALLBACK);

/* New Synchronous and thread-safe calls */
int  WINAPI SD_Init        ( LP_SDI_HANDLE );
int  WINAPI SD_Lock        ( SDI_HANDLE, SD_CHAR * );
int  WINAPI SD_Check       ( SDI_HANDLE, SD_CHAR *, SD_CHAR * );
int  WINAPI SD_ClientCheck ( SDI_HANDLE, SD_CHAR *, SD_CHAR *, SD_U32 );
int  WINAPI SD_Next        ( SDI_HANDLE, SD_CHAR * );
int  WINAPI SD_Pin         ( SDI_HANDLE, SD_CHAR * );
int  WINAPI SD_Close       ( SDI_HANDLE );

/* New Synchronous and thread-safe calls */
SD_ERROR WINAPI AceStartAuth(LP_SDI_HANDLE,
    SD_CHAR *userID,         
    SD_I32 userIDLen,     
    SD_BOOL *moreData,    
    SD_BOOL *noEcho,      
    SD_I32 *respTimeout,  
    SD_I32 *nextRespLen,  
    SD_CHAR *promptStr,      
    SD_I32 *promptStrLen);

SD_ERROR WINAPI AceContinueAuth(SDI_HANDLE, 
    SD_CHAR *userID,         
    SD_I32 userIDLen,     
    SD_BOOL *moreData,    
    SD_BOOL *noEcho,      
    SD_I32 *respTimeout,  
    SD_I32 *nextRespLen,  
    SD_CHAR *promptStr,      
    SD_I32 *promptStrLen);

SD_ERROR WINAPI AceCloseAuth(SDI_HANDLE);

#endif /* !USE_ACE_AGENT_API_PROTOTYPES */

#endif /* #ifndef WIN32 */

#ifdef  __cplusplus
}
#endif

#ifdef WIN32
#pragma pack(pop)
#endif

#endif /* !ACEXPORT_H */
