/*
 *   Copyright (C) 2002-2006
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef TACACS_H
#define TACACS_H

class tacacs
{
        TACACSSocket *sock;
        u_char *pak;
	HDR *hdr;

	db *dbh;
	file *fileh;

	friend class tauthen;
	friend class tauthor;
public:
	tacacs();
	~tacacs();
        void Execute(TACACSSocket *);

	void authen();
	void author();
	void accounting();
};

class tauthen {
	TACACSSocket *sock;
	db *dbh;
	file *fileh;
	int aborted;

	int state;		// Password state
	int action;		// What's being done
	int service;		// calling service
	int status;		// Authentication status
	int atype;		// Authentication type
	u_char flags;		// input and output flags

	string server_msg;
	string server_data;
	char *client_msg;
	char *client_data;
	int client_dlen;

	struct authen_type *type;
	struct identity *id;

	int choose_authen();
	int default_v0_fn();
	int default_fn();
	int enable_fn();
	int sendauth_fn();
	int sendpass_fn();
	int get_secret(string *secret);
	int verify(string *password, int auth_type);
	void access();
public:
	tauthen(tacacs *session);
	tauthen(TACACSSocket *s, db *sdb, file *sfile) {
		sock = s;
		fileh = sfile;
		dbh = sdb;
	        type = new struct authen_type;
	       	aborted = 0;
	        type->authen_func = 0;
	};
	~tauthen();
	void authenticate();
	int choose();
	friend class tacacs;
};


class tauthor {
	TACACSSocket *sock;
	db *dbh;
	file *fileh;
	int aborted;

	struct identity *id;
	struct author_user *user;
	int authen_method;		//Authentication method
	int authen_type;		//Authentication type
	int service;			//Calling service
	string *msg;			//Optional NULL-terminated return message
	string *admin_msg;		//Optional NULL-terminated admin message
	int status;			//Return status
	map<string,string> input_args;		//Input arguments
	map<string,string> output_args;		//Output arguments

public:

	tauthor(tacacs *session);
	~tauthor();
	int authorize();
	friend class tacacs;
	friend class db;

	int author_svc(map<string,string> cfg_args, int denybydefault);
	int pre_authorization(string cmd);
	void post_authorization(string cmd);
	int get_nas_svc(string *cmd, string *protocol, string *svc);
};


#endif
